/*
Copyright (c) 2015 - present Advanced Micro Devices, Inc. All rights reserved.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "CUDA2HIP.h"

// Maps CUDA header names to HIP header names
const std::map<llvm::StringRef, hipCounter> CUDA_DEVICE_FUNC_MAP {
  // math functions
  {"abs",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"labs",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"llabs",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fabs",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fabsf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"min",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fminf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fmin",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"max",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fmaxf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fmax",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sin",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cos",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sincos",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sincosf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"tan",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sqrt",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rsqrt",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rsqrtf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"exp2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"exp2f",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"exp10",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"exp10f",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"expm1",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"expm1f",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log2f",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log10",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log1p",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log1pf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"floor",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"exp",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cosh",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sinh",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"tanh",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"acosh",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"acoshf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"asinh",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"asinhf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atanh",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atanhf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ldexp",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ldexpf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"logb",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"logbf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ilogb",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ilogbf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"scalbn",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"scalbnf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"scalbln",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"scalblnf",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"frexp",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"frexpf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"round",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"roundf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"lround",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"lroundf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"llround",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"llroundf",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rint",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rintf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"lrint",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"lrintf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"llrint",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"llrintf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"nearbyint",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"nearbyintf",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ceil",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"trunc",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"truncf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fdim",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fdimf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atan2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atan",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"acos",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"asin",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hypot",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rhypot",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hypotf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rhypotf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"norm3d",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rnorm3d",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"norm4d",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rnorm4d",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"norm",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rnorm",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rnormf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"normf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"norm3df",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rnorm3df",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"norm4df",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rnorm4df",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cbrt",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cbrtf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rcbrt",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"rcbrtf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sinpi",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sinpif",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cospi",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cospif",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sincospi",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sincospif",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"pow",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"modf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fmod",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"remainder",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"remainderf",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"remquo",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"remquof",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"j0",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"j0f",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"j1",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"j1f",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"jn",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"jnf",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"y0",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"y0f",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"y1",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"y1f",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"yn",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ynf",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cyl_bessel_i0",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cyl_bessel_i0f",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cyl_bessel_i1",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cyl_bessel_i1f",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erf",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erff",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfinv",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfinvf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfc",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfcf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"lgamma",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfcinv",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfcinvf",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"normcdfinv",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"normcdfinvf",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"normcdf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"normcdff",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfcx",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"erfcxf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"lgammaf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"tgamma",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"tgammaf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"copysign",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"copysignf",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"nextafter",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"nextafterf",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"nan",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"nanf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fma",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fmaf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"acosf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"asinf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atanf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atan2f",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"cosf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sinf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"tanf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"coshf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sinhf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"tanhf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"expf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"logf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"log10f",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"modff",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"powf",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"sqrtf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"ceilf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"floorf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fmodf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"signbit",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"isfinite",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"isnan",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"isinf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"umin",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"llmin",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"ullmin",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"umax",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"llmax",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"ullmax",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__isinff",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__isnanf",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__finite",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__finitef",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__signbit",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__isnan",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__isinf",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__signbitf",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__signbitl",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__finitel",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__isinfl",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__isnanl",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"_ldsign",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"_fdsign",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"_Pow_int",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  // static math functions declared in device-functions.h
  {"mulhi",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"mul64hi",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"float_as_int",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"int_as_float",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"float_as_uint",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"uint_as_float",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"saturate",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"mul24",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"umul24",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"float2int",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"int2float",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"uint2float",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  // device functions
  {"__mulhi",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__umulhi",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__mul64hi",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__umul64hi",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int_as_float",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float_as_int",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint_as_float",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float_as_uint",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__syncthreads",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__threadfence",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__threadfence_block", {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__saturatef",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__sad",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__usad",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__mul24",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__umul24",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fdividef",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fdividef",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"fdivide",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__sinf",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__cosf",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__tanf",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__sincosf",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__expf",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__exp10f",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__log2f",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__log10f",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__logf",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__powf",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2int_rn",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2int_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2int_ru",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2int_rd",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2uint_rn",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2uint_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2uint_ru",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2uint_rd",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2float_rn",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2float_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2float_ru",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2float_rd",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2float_rn",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2float_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2float_ru",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2float_rd",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ll_rn",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ll_rz",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ll_ru",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ll_rd",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ull_rn",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ull_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ull_ru",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2ull_rd",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2float_rn",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2float_rz",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2float_ru",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2float_rd",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2float_rn",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2float_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2float_ru",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2float_rd",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fadd_rn",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fadd_rz",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fadd_ru",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fadd_rd",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsub_rn",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsub_rz",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsub_ru",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsub_rd",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmul_rn",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmul_rz",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmul_ru",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmul_rd",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmaf_rn",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmaf_rz",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmaf_ru",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fmaf_rd",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__frcp_rn",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__frcp_rz",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__frcp_ru",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__frcp_rd",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsqrt_rn",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsqrt_rz",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsqrt_ru",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fsqrt_rd",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__frsqrt_rn",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fdiv_rn",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fdiv_rz",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fdiv_ru",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__fdiv_rd",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__clz",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ffs",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__popc",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__brev",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__clzll",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ffsll",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__popcll",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__brevll",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__byte_perm",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hadd",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__rhadd",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uhadd",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__urhadd",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__double2int_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__double2uint_rz",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__double2ll_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__double2ull_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__prof_trigger",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__trap",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__brkpt",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__pm0",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__pm1",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__pm2",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__pm3",               {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  // SIMD functions
  {"__vabs2",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabsss2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vadd2",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vaddss2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vaddus2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vavgs2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vavgu2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vhaddu2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpeq2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpges2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpgeu2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpgts2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpgtu2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmples2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmplts2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpltu2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpne2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabsdiffu2",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vmaxs2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vmaxu2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vmins2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vminu2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vseteq2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetges2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetgeu2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetgts2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetles2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetleu2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetlts2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetltu2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetne2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsadu2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsub2",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsubss2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsubus2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vneg2",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vnegss2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabsdiffs2",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsads2",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabs4",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabsss4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vadd4",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vaddss4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vaddus4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vavgs4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vavgu4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vhaddu4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpeq4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpges4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpgeu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpgts4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpgtu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmples4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpleu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmplts4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpltu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vcmpne4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabsdiffu4",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vmaxs4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vmaxu4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vmins4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vminu4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vseteq4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetles4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetleu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetlts4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetltu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetges4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetgeu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetgts4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetgtu4",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsetne4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsadu4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsub4",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsubss4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsubus4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vneg4",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vnegss4",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vabsdiffs4",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__vsads4",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  // fp16 functions
  {"__float2half",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2half_rn",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2half_rz",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2half_rd",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2half_ru",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2float",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float2half2_rn",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__floats2half2_rn", {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__low2float",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__high2float",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__float22half2_rn", {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half22float2",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2int_rn",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2int_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2int_rd",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2int_ru",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2half_rn",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2half_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__int2half_ru",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2short_rn",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2short_rz",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2short_rd",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2short_ru",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__short2half_rn",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__short2half_rz",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__short2half_rd",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__short2half_ru",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2uint_rn",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2uint_rz",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2uint_rd",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2uint_ru",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2half_rn",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2half_rz",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2half_rd",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__uint2half_ru",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ushort_rn",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ushort_rz",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ushort_rd",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ushort_ru",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ushort2half_rn",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ushort2half_rz",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ushort2half_rd",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ushort2half_ru",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ull_rn",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ull_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ull_rd",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ull_ru",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2half_rn",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2half_rz",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2half_rd",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ull2half_ru",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ll_rn",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ll_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ll_rd",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2ll_ru",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2half_rn",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2half_rz",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2half_rd",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ll2half_ru",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"htrunc",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hceil",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hfloor",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hrint",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2trunc",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2ceil",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2floor",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2rint",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half2half2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__lowhigh2highlow", {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__lows2half2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__highs2half2",     {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__high2half",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__low2half",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hisinf",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__halves2half2",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__low2half2",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__high2half2",      {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half_as_short",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__half_as_ushort",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__short_as_half",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ushort_as_half",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ldg",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ldcg",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ldca",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__ldcs",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__heq2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hne2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hle2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hge2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hlt2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hgt2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hequ2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hneu2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hleu2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hgeu2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hltu2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hgtu2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hisnan2",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hadd2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hsub2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hmul2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__h2div",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hadd2_sat",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hsub2_sat",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hmul2_sat",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hfma2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hfma2_sat",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hneg2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hsub",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hmul",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hdiv",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hadd_sat",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hsub_sat",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hmul_sat",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hfma",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hfma_sat",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hneg",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__habs2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__habs",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__hbeq2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbne2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hble2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbge2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hblt2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbgt2",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbequ2",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbneu2",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbleu2",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbgeu2",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbltu2",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hbgtu2",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__heq",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hne",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hle",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hge",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hlt",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hgt",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hequ",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hneu",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hleu",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hgeu",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hltu",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hgtu",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__hisnan",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hsqrt",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hrsqrt",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hrcp",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hlog",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hlog2",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hlog10",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hexp",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hexp2",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hexp10",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hcos",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"hsin",              {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2sqrt",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2rsqrt",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2rcp",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2log",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2log2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2log10",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2exp",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2exp2",            {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2exp10",           {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2cos",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"h2sin",             {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"__shfl_sync",       {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__shfl_up_sync",    {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__shfl_down_sync",  {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  {"__shfl_xor_sync",   {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1, UNSUPPORTED}},
  // atomic functions
  {"atomicAdd",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicSub",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicExch",        {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicMin",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicMax",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicInc",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicDec",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicAnd",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicOr",          {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicXor",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
  {"atomicCAS",         {"", "", CONV_DEVICE_FUNC, API_RUNTIME, 1}},
};

const std::map<llvm::StringRef, cudaAPIversions> CUDA_DEVICE_FUNC_VER_MAP {
};

const std::map<unsigned int, llvm::StringRef> CUDA_DEVICE_FUNC_SECTION_MAP {
  {1, "Device Functions"},
};
